# Train Keras MLP example

Keras models can be trained simply by running them.

    >>> cd("examples/keras")

The keras plugin repsonsible for running Keras scripts automatically
adds a model checkpoint callback to save model weights at each
epoch. However, this callback requires the `h5py` module, which isn't
installed by default.

For our first training, we'll proceed without the `h5py` module.

    >>> run("guild run mnist_mlp.py -y --no-gpus epochs=1",
    ...     timeout=180, ignore=("WARNING", "Refreshing"))
    Limiting available GPUs (CUDA_VISIBLE_DEVICES) to: <none>
    Using TensorFlow backend.
    60000 train samples
    10000 test samples...
    _________________________________________________________________
    Layer (type)                 Output Shape              Param #
    =================================================================
    dense_1 (Dense)              (None, 512)               401920
    _________________________________________________________________
    dropout_1 (Dropout)          (None, 512)               0
    _________________________________________________________________
    dense_2 (Dense)              (None, 512)               262656
    _________________________________________________________________
    dropout_2 (Dropout)          (None, 512)               0
    _________________________________________________________________
    dense_3 (Dense)              (None, 10)                5130
    =================================================================
    Total params: 669,706
    Trainable params: 669,706
    Non-trainable params: 0
    _________________________________________________________________
    ...Train on 60000 samples, validate on 10000 samples...
    Epoch 1/1
    <BLANKLINE>
    ...
    Test loss: ...
    Test accuracy: ...
    <exit 0>

Note the warning from the keras plugin.

Here are the files generated by the run:

    >>> run("guild runs info --files")
    id: ...
    files:
      logs
      logs/events.out.tfevents...
    <exit 0>

And using `ls`:

    >>> run("echo dummy && guild ls")
    dummy
    ...:
      logs/
      logs/events.out.tfevents...
    <exit 0>

We see that the run generated a TensorBoard event log but did not save
any model weights.

Let install `h5py`:

    >>> quiet("guild install h5py")

And train again:

    >>> run("guild run -y --no-gpus mnist_mlp.py epochs=1", timeout=120,
    ...     ignore=["FutureWarning", "_register_converters", "WARNING"])
    Limiting available GPUs (CUDA_VISIBLE_DEVICES) to: <none>
    Using TensorFlow backend.
    60000 train samples
    10000 test samples...
    _________________________________________________________________
    Layer (type)                 Output Shape              Param #
    =================================================================
    dense_1 (Dense)              (None, 512)               401920
    _________________________________________________________________
    dropout_1 (Dropout)          (None, 512)               0
    _________________________________________________________________
    dense_2 (Dense)              (None, 512)               262656
    _________________________________________________________________
    dropout_2 (Dropout)          (None, 512)               0
    _________________________________________________________________
    dense_3 (Dense)              (None, 10)                5130
    =================================================================
    Total params: 669,706
    Trainable params: 669,706
    Non-trainable params: 0
    _________________________________________________________________
    ...Train on 60000 samples, validate on 10000 samples...
    Epoch 1/1
    <BLANKLINE>
    ...
    Test loss: ...
    Test accuracy: ...
    <exit 0>

Note the warning is no longer printed.

Here are the generated files:

    >>> run("guild runs info --files")
    id: ...
    files:
      logs
      logs/events.out.tfevents...
    <exit 0>

Here we see the TensorBoard logs and also the model weights.

We need to remove `h5py` as it can noisily log warnings, which we
don't want to see in future tests.

    >>> quiet("pip uninstall h5py -y")
